package orchid.agent;

import orchid.object.*;
import java.util.*;
import java.io.*;
import com.objectspace.voyager.mobility.*;
import com.objectspace.voyager.*;
import com.objectspace.lib.*;
import com.objectspace.lib.util.*;
import com.objectspace.lib.facets.*;
import com.objectspace.voyager.agent.*;
import com.objectspace.voyager.space.*;
import com.objectspace.voyager.space.Subspace;

/**
 * Engine phase 02 for the MeetingAgent.
 * @author Marc-Aurle DARCHE
 */
public class MAEngine02 extends Thread {
	private Calendar endtime;
	private MeetingAgent father;
	private MeetingProposal mp;
	private int rythm;
	private int CF= 0;
	private Vector meetingList;
	boolean dhp_accepted;

	public MAEngine02(MeetingAgent father, MeetingProposal mp, Calendar endtime, int rythm) {
		this.endtime = endtime;
		this.father = father;
		this.mp = mp;
		this.rythm = rythm;
	}

	public void run() {
		Hashtable guestinfolist = mp.getGuestInfoList();
		Enumeration e;

		orchid.object.Util.logMsg("info","MA-Engine02 working on meeting "+mp.getKey(),"MA-Engine02 launched");
		this.setPriority(MIN_PRIORITY);

		// Retrieve meetingList of father to compute CF.
		meetingList= father.getMeetings(mp.getBetweenDate(),mp.getAndDate());
		int meetingListSize=meetingList.size();
		CF= meetingListSize;

		try {
			//ISubspace meetingsubspace = (ISubspace) Namespace.lookup("//"+father.getCurrentlocation()+"/"+mp.getKey());
			ISubspace meetingsubspace = (ISubspace) Namespace.lookup("//"+father.getCurrentlocation()+"/Room."+mp.getInitiator());
			IMeetingAgent maofthemeeting = (IMeetingAgent) meetingsubspace.getMulticastProxy("orchid.agent.IMeetingAgent");
			maofthemeeting.giveYouMyCF(mp.getKey(),CF,father.agent_user.getName());
		}
		catch (Exception ex) {
			orchid.object.Util.logMsg("error","MAEngine02","Meeting Subspace can't be reached "+ex);
		}

		while ( father.numofcfreceived < (guestinfolist.size()-1) ) { // Initiator doesn't compute and send its CF
			try {
				this.sleep(rythm);
			} catch (Exception exx) {
				orchid.object.Util.logMsg("error","MAEngine02","Can't sleep"+exx);
			}
			orchid.object.Util.logMsg("info","MAEngine02 ("+father.agent_user.getName()+")",father.numofcfreceived+" CF received");
		}

		// obtain the CF list sorted
		Hashtable cftable = new Hashtable();
		Enumeration en = mp.getGuestInfoList().keys();
		while (en.hasMoreElements())
		{
			String name = (String) en.nextElement();
			if (!name.equals(mp.getInitiator()))
				cftable.put(name, new Integer(((GuestInfo) mp.getGuestInfoList().get(name)).getCF()));
		}
		Vector cfvector  = orchid.object.Util.sortCF(cftable);
		// find the next MCP agent
		int k = cfvector.indexOf(father.getOwner());
		int l = k;
		if ( k < (cfvector.size()-1) )
			k++;
		String nextmcp = new String();
		nextmcp = (String) cfvector.elementAt(k);
		if ( l > 0 )
			l--;
		String prevmcp = new String();
		prevmcp = (String) cfvector.elementAt(l);
		orchid.object.Util.logMsg("info","MAEngine02 ("+father.agent_user.getName()+")","the prev MCP to me is "+prevmcp);
		orchid.object.Util.logMsg("info","MAEngine02 ("+father.agent_user.getName()+")","the next MCP to me is "+nextmcp);

		DateHourProposal dhp=new DateHourProposal();
		Calendar timeBetween = Calendar.getInstance();
		Calendar timeAnd = (Calendar)timeBetween.clone();
		timeBetween.setTime(mp.getBetweenDate());
		timeAnd.setTime(mp.getAndDate());

		orchid.object.Util.logMsg("info","MAEngine02 ("+father.agent_user.getName()+")","mp.getRangeLength() = "+mp.getRangeLength());
		RangeProposal rp= new RangeProposal(mp.getRangeLength());

		while ( ! father.nomore ) {
			// test wether the agent is the MCP and sets father.kMCP_contacted accordingly
			if ( father.agent_user.getName().equals(cfvector.firstElement()) ) {
			  // dhp generation
			  //orchid.object.Util.logMsg("info","MAEngine02 ("+father.agent_user.getName()+")","COMPARE "+timeBetween.getTime().toString()+" vs "+timeAnd.getTime().toString());
				if (timeBetween.before(timeAnd)) { 
					int year=timeBetween.get(Calendar.YEAR);
					int month=timeBetween.get(Calendar.MONTH);
					int day=timeBetween.get(Calendar.DAY_OF_MONTH);
					int timeOfDay=800;
					dhp.setDate(year, month, day);
					dhp.setTimeOfDay(timeOfDay);
					orchid.object.Util.logMsg("info","MAEngine02 ("+father.agent_user.getName()+")","Generation (<) of "+dhp);
				} else {
					if (timeBetween.equals(timeAnd)) {
					  int year=timeBetween.get(Calendar.YEAR);
					  int month=timeBetween.get(Calendar.MONTH);
					  int day=timeBetween.get(Calendar.DAY_OF_MONTH);
					  int timeOfDay=800;
					  dhp.setDate(year, month, day);
					  dhp.setTimeOfDay(timeOfDay);
					  orchid.object.Util.logMsg("info","MAEngine02 ("+father.agent_user.getName()+")","Generation (=)  of "+dhp);
					}
					else {
					  father.noMoreProposal(mp.getKey());
					}
				}
			} else {
				// waiting to receive dhp from previous MCP (aka kMCP)
				while (! (father.kMCP_contacted || father.nomore) ) {
					try {
						//this.sleep(rythm);
						this.sleep(10);
					} catch (Exception exx) {
						orchid.object.Util.logMsg("error","MAEngine02 ("+father.agent_user.getName()+")","Can't sleep"+exx);
					}
				}//end of while
				father.kMCP_contacted= false; // once contacted set the flag to previous position
				dhp = (DateHourProposal) father.dateHourProposalList.get(mp.getKey());
			}//end of else

			//comparison
			Calendar caltotest = Calendar.getInstance();
			caltotest.set(Calendar.YEAR, dhp.getYear());
			caltotest.set(Calendar.MONTH, dhp.getMonth());
			caltotest.set(Calendar.DAY_OF_MONTH, dhp.getDay());
			Vector temp_meetingList = father.getMeetings(caltotest.getTime(),caltotest.getTime());
			if ( temp_meetingList.size()==0 )
			  dhp_accepted=true;
			else dhp_accepted=false;
			
			
			if (!father.nomore) {
			if (dhp_accepted) {
			if ( ! father.agent_user.getName().equals(cfvector.lastElement()) ) {
			  // if not the nMCP
				try {
					Vector v= new Vector();
					v.addElement(nextmcp);
					Vector agentslist = father.getAgentsUrl(v);
					IMeetingAgent nextAgent_ma = (IMeetingAgent) Namespace.lookup("//"+((AgentInfo)agentslist.elementAt(0)).getCurrentlocation()+"/"+((AgentInfo)agentslist.elementAt(0)).getName());
					nextAgent_ma.newProposal(mp.getKey(), dhp);
				} catch (NamespaceException nsex) {
					orchid.object.Util.logMsg("error","MAEngine02 ("+father.agent_user.getName()+")","Can't find next MCP agent.");
				}

				// wait for next request signal from next MCP
				while ( ! father.next ) {
					try {
						this.sleep(10);
					} catch (Exception exx) {
						orchid.object.Util.logMsg("error","MAEngine02 ("+father.agent_user.getName()+")","Can't sleep"+exx);
					}
				}//end of while
				//orchid.object.Util.logMsg("info","MAEngine02 ("+father.agent_user.getName()+")",nextmcp+" wants another dhp from me");
				father.next=false;
				
			} else {
			  // if nMCP
				try {
				  //orchid.object.Util.logMsg("info","MAEngine02 ("+father.agent_user.getName()+")","filing the RP with dhp : "+dhp);
				  //orchid.object.Util.logMsg("info","MAEngine02 ("+father.agent_user.getName()+")","mp.getBetweenDate() : "+mp.getBetweenDate());

				  rp.setMeeting(mp.getBetweenDate(),dhp,mp.getDuration());
				  //orchid.object.Util.logMsg("info","MAEngine02 ("+father.agent_user.getName()+")","RP set to "+rp.toString());
				} catch (IOException ioe) {
					orchid.object.Util.logMsg("error","MAEngine02 ("+father.agent_user.getName()+")","IOException de la part de MAD");
				  } 

				try {
					Vector v= new Vector();
					v.addElement(prevmcp);
					Vector agentslist = father.getAgentsUrl(v);
					IMeetingAgent prevAgent_ma = (IMeetingAgent) Namespace.lookup("//"+((AgentInfo)agentslist.elementAt(0)).getCurrentlocation()+"/"+((AgentInfo)agentslist.elementAt(0)).getName());
					prevAgent_ma.nextProposal(mp.getKey());
				} catch (NamespaceException nsex) {
					orchid.object.Util.logMsg("error","MAEngine02 ("+father.agent_user.getName()+")","Can't find previous MCP agent.");
				}
				/*
				try {
					Vector v= new Vector();
					v.addElement(mp.getInitiator());
					Vector agentslist = father.getAgentsUrl(v);
					IMeetingAgent initiator_ma = (IMeetingAgent) Namespace.lookup("//"+((AgentInfo)agentslist.elementAt(0)).getCurrentlocation()+"/"+((AgentInfo)agentslist.elementAt(0)).getName());
				} catch (NamespaceException nsex) {
					orchid.object.Util.logMsg("info","MAEngine02 ("+father.agent_user.getName()+")","Can't find initiator agent.");
				}
				initiator_ma.getInitiator().giveResult(String meetingProposalKey, RangeProposal rp)
				*/
			}
			} else {
			  try {
				Vector v= new Vector();
				v.addElement(prevmcp);
				Vector agentslist = father.getAgentsUrl(v);
				IMeetingAgent prevAgent_ma = (IMeetingAgent) Namespace.lookup("//"+((AgentInfo)agentslist.elementAt(0)).getCurrentlocation()+"/"+((AgentInfo)agentslist.elementAt(0)).getName());
				prevAgent_ma.nextProposal(mp.getKey());
			  } catch (NamespaceException nsex) {
				orchid.object.Util.logMsg("error","MAEngine02 ("+father.agent_user.getName()+")","Can't find prev MCP agent.");
			  }
			}
			}
			
			if ( father.agent_user.getName().equals(cfvector.firstElement()) )
				timeBetween.add(Calendar.DATE,1);
			//timeBetween.add(timeBetween.DATE,1);
			//orchid.object.Util.logMsg("info","MAEngine02 ("+father.agent_user.getName()+")","father.nomore : "+father.nomore);

		}// end of while (nomore loop)	
		father.nomore= false; // once no more dhp to propose
		try {
		  Vector v= new Vector();
		  v.addElement(nextmcp);
		  Vector agentslist = father.getAgentsUrl(v);
		  IMeetingAgent nextAgent_ma = (IMeetingAgent) Namespace.lookup("//"+((AgentInfo)agentslist.elementAt(0)).getCurrentlocation()+"/"+((AgentInfo)agentslist.elementAt(0)).getName());
		  nextAgent_ma.noMoreProposal(mp.getKey());
		} catch (NamespaceException nsex) {
		  orchid.object.Util.logMsg("info","MAEngine02 ("+father.agent_user.getName()+")","Can't find initiator agent.");
		}
		if ( father.agent_user.getName().equals(cfvector.lastElement()) )
		  orchid.object.Util.logMsg("info","MAEngine02 ("+father.agent_user.getName()+")","the rp is "+rp.toString());

		orchid.object.Util.logMsg("info","MAEngine02 ("+father.agent_user.getName()+")","rp size is : "+rp.getRangeSize());
		orchid.object.Util.logMsg("info","MAEngine02 ("+father.agent_user.getName()+")","*** MAEngine02 terminated ***\n");
	}// end of run()

}

