package orchid.object;

import java.util.*;

/**
 * A class used to propose a date and an hour for a meeting 
 * a user has been invited to.<BR>
 * A stored date hour proposal looks like:<P>
 * <TT>
 * Fri Oct 02 00:00:00 GMT+00:00 1998 1400 
 * </TT>
 * <P>The objects to arrange a meeting are used in this order :
 * <ol>
 * <li> MeetingProposal
 * <li> DateHourProposal
 * <li> RangeProposal
 * <li> Meeting
 * </ol>
 * @see MeetingProposal
 * @see RangeProposal
 * @see Meeting
 * @see ObjectTest
 * @author Marc-Aurle DARCHE
 */
public class DateHourProposal implements java.io.Serializable{

	private Date aDate;	// yyyy-mm-dd
	private int timeOfDay;  // hh30 or hh00

	public DateHourProposal() {
	}

	public DateHourProposal(int year, int month, int day, int timeOfDay) {
		Calendar rightNow = Calendar.getInstance();
		rightNow.set(year,month,day,0,0,0);
		this.aDate=rightNow.getTime();
		this.timeOfDay=timeOfDay;
	}

	public void setDate(int year, int month, int day) {
		Calendar rightNow = Calendar.getInstance();
		rightNow.set(year,month,day,0,0,0);
		this.aDate=rightNow.getTime();
	}

	public void setTimeOfDay(int timeOfDay) {
		this.timeOfDay=timeOfDay;
	}

	public Date getDate() {
		return this.aDate;
	}

	public int getYear() {
		Calendar rightNow = Calendar.getInstance();
		rightNow.setTime(this.aDate);
		return rightNow.get(Calendar.YEAR);
	}

	public int getMonth() {
		Calendar rightNow = Calendar.getInstance();
		rightNow.setTime(this.aDate);
		return rightNow.get(Calendar.MONTH);
	}

	public int getDay() {
		Calendar rightNow = Calendar.getInstance();
		rightNow.setTime(this.aDate);
		return rightNow.get(Calendar.DATE);
	}

	public int getTimeOfDay() {
		return this.timeOfDay;
	}

	public String toString() {
		return aDate.toString()+" "+new Integer(timeOfDay).toString();
	}

}
