package orchid.server;

import orchid.object.*;
import orchid.agent.*;
import com.objectspace.voyager.*;
import com.objectspace.voyager.mobility.*;
import com.objectspace.lib.util.*;

/**
 * The purpose of this class is to create agent on servers, so they can work.
 * @author Marc-Aurle DARCHE
 */
public class AgentCreation{

	/** voyager level of verbosity: no output */
	public static final int level1=Console.SILENT;
	/** voyager level of verbosity: stack traces of remote and unhandled exceptions */
	public static final int level2=Console.EXCEPTIONS;
	/** voyager level of verbosity: same as previous + internal debug information */
	public static final int level3=Console.VERBOSE;

	//private String defaultServer= "192.168.1.105:10000";
	private String defaultServer= "127.0.0.1:10000";
	
	public AgentCreation(String defaultServer){
		System.out.println("\n*** Beginning of AgentCreation ***\n");
		this.defaultServer=defaultServer;
		
		String classname="orchid.agent.MeetingAgent";
		
		String name1="philippe.benhamou";
		UserInfo u1= new UserInfo(name1,defaultServer);
		Object[] wrapped_args1= new Object[]{u1};
		
		String name2="antoine.souliez";
		UserInfo u2= new UserInfo(name2,defaultServer);
		Object[] wrapped_args2= new Object[]{u2};
		
		String name3="ma.darche";
		UserInfo u3= new UserInfo(name3,defaultServer);
		Object[] wrapped_args3= new Object[]{u3};

		try {
			System.out.println(" -- AgentCreation.class: Starting Voyager ...");
			Console.setLogLevel(level2);
			Voyager.startup();
		} catch(Exception exception) {System.err.println("AgentCreation.class: Voyager startup >>> "+exception);}
	
		ISaa saa;
		boolean saapresent = false;

		try {
			saa = (ISaa) Namespace.lookup("//"+defaultServer+"/SAA");
			saapresent = true;

			System.out.println(" -- AgentCreation.class: Beginning of creation process");
			IMeetingAgent m1agent = (IMeetingAgent) Factory.create(classname,wrapped_args1,"//"+defaultServer);
			if (saapresent)
				{
					AgentInfo agenti = new AgentInfo("ma__"+u1.getName(),u1.getName(),u1.getHome(),u1.getHome(), false);
					saa.addAgent(agenti);
				}
			IMeetingAgent m2agent = (IMeetingAgent) Factory.create(classname,wrapped_args2,"//"+defaultServer);
			if (saapresent)
				{
					AgentInfo agenti = new AgentInfo("ma__"+u2.getName(),u2.getName(),u2.getHome(),u2.getHome(), false);
					saa.addAgent(agenti);
				}
			IMeetingAgent m3agent = (IMeetingAgent) Factory.create(classname,wrapped_args3,"//"+defaultServer);
			if (saapresent)
				{
					AgentInfo agenti = new AgentInfo("ma__"+u3.getName(),u3.getName(),u3.getHome(),u3.getHome(), false);
					saa.addAgent(agenti);
				}
			System.out.println(" -- AgentCreation.class: Successful Factory create");
			/*
			We don't need to obtain the Mobility facets anymore since 
			the Meeting Agents implements the 2 moveTo Methods !!!
			But if something goes wrong, IMobility has to be DE-implemented since
			there seems to be a bug in this Interface (check MeetingAgent javadoc for more info) !!!
			*/
			/*
			IMobility mobility1 = Mobility.of(m1agent);
			IMobility mobility2 = Mobility.of(m2agent);
			IMobility mobility3 = Mobility.of(m3agent);
			System.out.println(" -- AgentCreation.class: Successful obtaining of the Mobility facets of the agents");
			*/
			Namespace.bind("//"+defaultServer+"/ma__"+name1,m1agent);
			Namespace.bind("//"+defaultServer+"/ma__"+name2,m2agent);
			Namespace.bind("//"+defaultServer+"/ma__"+name3,m3agent);
			System.out.println(" -- AgentCreation.class: Successful Bind");
		} catch(Exception exception) {System.err.println("AgentCreation.class: Trying to create agents on server "+defaultServer+" >>> "+exception);}
		System.out.println("\n*** End of AgentCreation ***\n");
	}

	public static void main(String[] args){
		String serverstring="127.0.0.1:10000";
		if (args.length>0) { serverstring= args[0]; }
		AgentCreation ac = new AgentCreation(serverstring);
	}
}

