package orchid.server;

import java.io.*;
import java.util.*;
import java.net.*;
import com.objectspace.voyager.*;
import com.objectspace.voyager.space.*;
import com.objectspace.voyager.space.Subspace.*;
import com.objectspace.lib.*;
import com.objectspace.lib.util.*;
import com.objectspace.voyager.space.multicasting.*;

import orchid.agent.*;
import orchid.object.*;

/**
 * Launch a server and do some tests.<P>
 * Syntax is :<BR>
 * java orchid.server.Boot <self server> <master server><P>
 * <P>
 * Examples :<BR>
 * <ul>
 * <li><b><TT>java orchid.server.Boot</TT></b> <BR>
 * will start a Master server with IP@ 127.0.0.1:10000
 * <li><b><TT>java orchid.server.Boot 192.168.1.105:10000</b></TT> <BR>
 * will start a Master server with IP@ 192.168.1.105:10000
 * <li><b><TT>java orchid.server.Boot 192.168.1.105:10000 192.168.1.103:10001</b></TT> <BR>
 * will start a server with its IP@ 192.168.1.105:10000 and will connect it to a master server with IP@ 192.168.1.103:10001
 * </ul>
 * @author Antoine SOULIEZ
 */
public class Boot extends Thread
	{

		private static Server serveur;
		private static boolean serverisrunning = false;
		private static int choicemade = 100;
		private static String selfserv = "127.0.0.1:10000";
		private static String masterserv = "127.0.0.1:10000";

		public static void main (String[] args)
			{

				switch(args.length)
					{
						case 0 : break;
						case 1 : masterserv = args[0]; selfserv=args[0] ;break;
						case 2 : selfserv = args[0]; masterserv = args[1];break;
						default : System.out.println("Wrong number of arguments ! You may enter 0, 1 or 2 parameters\nSyntax is : java orchid.server.Boot\n or         java orchid.server.Boot <master server>\n or         java orchid.server.Boot <self server> <master server>\nExample : java orchid.server.Boot 192.168.1.105:10000 192.168.1.103:11000");System.exit(0); break;
					}
					System.out.println(args.length+" arguments, assuming\nself server as "+selfserv+"\nmaster server as "+masterserv);

				/*****************************************/

					System.out.println("------ CONTROL PAD ------\n");
					System.out.println("1 - Launch the server <Antoine S.>");
					System.out.println("2 - AgentCreation <Darche MA.>");
					System.out.println("3 - MP Creation on ma__antoine.souliez <Antoine S.>");
					System.out.println("4 - <empty slot>");
					System.out.println("5 - <empty slot>");
					System.out.println("6 - <empty slot>");
					System.out.println("7 - <empty slot>");
					System.out.println("8 - <empty slot>");
					System.out.println("9 - The Whole Deck");
					System.out.println("\n0 - Exiting the test area");
					System.out.print("\nEnter your choice : ");
					while (choicemade != 0)
					{
						try
						{
							choicemade = System.in.read()-48;
							int sertarien = System.in.read();
						} catch (IOException ioex) {};
						switch (choicemade)
						{
							case 1 : System.out.println("Test 1");if (serverisrunning) System.out.println("Server is already running"); else  test1(); break;
							case 2 : System.out.println("Test 2"); test2(); break;
							case 3 : System.out.println("Test 3"); test3(); break;
							case 4 : System.out.println("Test 4"); test4(); break;
							case 5 : System.out.println("Test 5"); test5(); break;
							case 6 : System.out.println("Test 6"); test6(); break;
							case 7 : System.out.println("Test 7"); test7(); break;
							case 8 : System.out.println("Test 8"); test8(); break;
							case 9 : System.out.println("Test 9"); test9(); break;
							case 0 : System.out.println("Test 0"); test0(); break;
							default : System.out.println("- Wrong choice- "+choicemade);break;
						}

						choicemade = 100;
					}
			}

		private static void test0()
		{
			if (serverisrunning)
			{
				serveur.serverDisconnect();
				serveur.serverStop();
			}
			System.out.println("Exiting test area");
			System.exit(0);
		}


		private static void test1()
		{
			ServerConfig srvcfg = new ServerConfig(selfserv, masterserv, "SSERV","GSERV","SAGENTS","GAGENTS", "orchid/log");
			serveur = new Server(srvcfg);
			serveur.run();
			serverisrunning = true;
		}

		private static void test2()
		{
			AgentCreation agc = new AgentCreation(selfserv);
		}

		private static void test3()
		{
			try {
				IMeetingAgent ma1 = (IMeetingAgent) Namespace.lookup("//"+selfserv+"/ma__antoine.souliez");
				if (ma1 != null)
				{
					Hashtable glist = new Hashtable();
					glist.put("antoine.souliez",new GuestInfo("antoine.souliez",false));
					glist.put("ma.darche",new GuestInfo("ma.darche",false));
					glist.put("philippe.benhamou",new GuestInfo("philippe.benhamou",false));
					MeetingProposal mp1 = new MeetingProposal("rien","pas mieux","antoine.souliez",1999,0,18,1999,01,01,400,glist,1);

					System.out.println("Calling callMeeting on a agent");
					ma1.callMeeting(mp1);
				}
			} catch (Exception ex41)
			{
				System.err.println("<ERROR from Boot> Unbale to reach ma1 "+ex41);
			}


		}
		private static void test4()
		{
		}

		private static void test5()
		{
		}

		private static void test6()
		{
		}

		private static void test7()
		{
		}

		private static void test8()
		{
		}

		private static void test9()
		{
			test1();
			test2();
			test3();
		}



	}

