package orchid.server;

import java.sql.*;

/**
 * A class used to delete, create and populate the SQL tables used by the <b>Orchid</b> system.<P>
 * Deletion : 			DBManipulation 0 <BR>
 * Creation :		 	DBManipulation 1 <BR>
 * Creation and Population : 	DBManipulation 2 (without servers and agents population)<P>
 * Creation and Population : 	DBManipulation 3 (with servers and agents population)<P>
 * Use Deletion before any Creation !
 * @author Marc-Aurle DARCHE
 */
public class DBManipulation {

static public void main(String args[]) {
	Statement stmt;

	if (args.length>0) {
	try{
		Class.forName("postgresql.Driver");
		//System.out.println("Class.forName OK!");

		String url="jdbc:postgresql://localhost:5432/orchid_db";
		String usr="pr9";
		String pwd="";
		Connection db = DriverManager.getConnection(url, usr, pwd);
		if(db != null){
			System.out.println("Connected to db");
		}else{System.out.println("Can't connect to db");}

		stmt = db.createStatement();

		switch (Integer.parseInt(args[0])){

		case 0:
			/* Tables deletion from the orchid DataBase */
			stmt.executeUpdate("drop table servers");
			System.out.println("table servers dropped !");
			stmt.executeUpdate("drop table users");
			System.out.println("table users dropped !");
			stmt.executeUpdate("drop table meetings");
			System.out.println("table meetings dropped !");
			stmt.executeUpdate("drop table agents");
			System.out.println("table agents dropped !");
			System.out.println("\ntables successfuly deleted !\n");
			break;

		case 1:
			/* Tables creation into the orchid DataBase */
			stmt.executeUpdate("create table servers" +
			" (name text, url text, availability int, master boolean)");

			stmt.executeUpdate("create table users (name text, password text," + 
			" info text, profile text, online boolean)");

			stmt.executeUpdate("create table meetings" +
			" (user text, subject text, note text, initiator text," + 
			" date date, timeofday int, duration int," + 
			" participants text, frozen boolean)");

			stmt.executeUpdate("create table agents" +
			 " (name text, owner text, home text, currentlocation text, frozen boolean)");
			System.out.println("\ntables successfuly created !\n");
			break;

		case 2:
			/* Tables creation into the orchid DataBase */
			stmt.executeUpdate("create table servers" +
			" (name text, url text, availability int, master boolean)");

			stmt.executeUpdate("create table users (name text, password text," + 
			" info text, profile text, online boolean)");

			stmt.executeUpdate("create table agents" +
			 " (name text, owner text, home text, currentlocation text, frozen boolean)");

			stmt.executeUpdate("create table meetings" +
			" (user text, subject text, note text, initiator text," + 
			" date date, timeofday int, duration int," + 
			" participants text, frozen boolean)");

			/* Populating the tables */
			stmt.executeUpdate("insert into users values ('philippe.benhamou','a23q','ONERA','',TRUE)");
			stmt.executeUpdate("insert into users values ('antoine.souliez','q$;s','ESIEA','',TRUE)");
			stmt.executeUpdate("insert into users values ('ma.darche','8;az','ESIEA','',TRUE)");

			stmt.executeUpdate("insert into meetings values" + 
			" ('ma.darche','installation NIS et CVS','Quelle version de NIS doit-on utiliser ?','ma.darche'," + 
			"'30/01/1999',1030,600," +
			 "'ma.darche antoine.souliez',FALSE)");

			stmt.executeUpdate("insert into meetings values" + 
			" ('ma.darche','Qui suis-je ?','Mais oui au fait ?','ma.darche'," + 
			"'30/01/1999',1930,130," +
			 "'ma.darche',FALSE)");

			stmt.executeUpdate("insert into meetings values" + 
			" ('philippe.benhamou','runion TPED','dfinition du terme agent','ma.darche'," + 
			"'1/10/1998',1430,400," +
			 "'philippe.benhamou ma.darche emmanuel.pierre frdric.salvy antoine.souliez mo.varroy',FALSE)");

			stmt.executeUpdate("insert into meetings values" + 
			" ('antoine.souliez','runion TPED','dfinition du terme agent','ma.darche'," + 
			"'1/10/1998',1430,400," +
			 "'philippe.benhamou ma.darche emmanuel.pierre frdric.salvy antoine.souliez mo.varroy',FALSE)");

			stmt.executeUpdate("insert into meetings values" + 
			" ('antoine.souliez','installation NIS et CVS','Quelle version de NIS doit-on utiliser ?','ma.darche'," + 
			"'30/01/1999',1030,600," +
			 "'ma.darche antoine.souliez',FALSE)");

			stmt.executeUpdate("insert into meetings values" + 
			" ('ma.darche','runion TPED','dfinition du terme agent','ma.darche'," + 
			"'1/10/1998',1430,400," +
			 "'philippe.benhamou ma.darche emmanuel.pierre frdric.salvy antoine.souliez mo.varroy',FALSE)");

			stmt.executeUpdate("insert into meetings values" + 
			" ('ma.darche','Fred y fait les choses  moiti','Et encore on est gentils','ma.darche'," + 
			"'18/02/1999',1430,400," +
			 "'philippe.benhamou ma.darche antoine.souliez',FALSE)");
			 
			System.out.println("\ntables successfuly created and populated !\n");
			break;

		case 3:
			/* Tables creation into the orchid DataBase */
			stmt.executeUpdate("create table servers" +
			" (name text, url text, availability int, master boolean)");

			stmt.executeUpdate("create table users (name text, password text," + 
			" info text, profile text, online boolean)");

			stmt.executeUpdate("create table agents" +
			 " (name text, owner text, home text, currentlocation text, frozen boolean)");

			stmt.executeUpdate("create table meetings" +
			" (user text, subject text, note text, initiator text," + 
			" date date, timeofday int, duration int," + 
			" participants text, frozen boolean)");

			/* Populating the tables */
			stmt.executeUpdate("insert into servers values ('i5.esiea.fr','192.168.1.105',10, TRUE)");
			stmt.executeUpdate("insert into servers values ('n7.esiea.fr','192.168.1.170',10, FALSE)");
			stmt.executeUpdate("insert into servers values ('nef.esiea.fr','192.168.1.103',4, FALSE)");

			stmt.executeUpdate("insert into users values ('philippe.benhamou','a23q','ONERA','',TRUE)");
			stmt.executeUpdate("insert into users values ('antoine.souliez','q$;s','ESIEA','',TRUE)");
			stmt.executeUpdate("insert into users values ('ma.darche','8;az','ESIEA','',TRUE)");

			stmt.executeUpdate("insert into agents values ('ma__philippe.benhamou','philippe.benhamou','192.168.1.105','192.168.1.170',TRUE)");
			stmt.executeUpdate("insert into agents values ('ma__antoine.souliez','antoine.souliez','192.168.1.105','192.168.1.170',FALSE)");
			stmt.executeUpdate("insert into agents values ('ma__ma.darche','ma.darche','192.168.1.105','192.168.1.105',FALSE)");


			stmt.executeUpdate("insert into meetings values" + 
			" ('ma.darche','installation NIS et CVS','Quelle version de NIS doit-on utiliser ?','ma.darche'," + 
			"'30/01/1999',1030,6," +
			 "'ma.darche antoine.souliez',FALSE)");

			stmt.executeUpdate("insert into meetings values" + 
			" ('philippe.benhamou','runion TPED','dfinition du terme agent','ma.darche'," + 
			"'1/10/1998',1430,4," +
			 "'philippe.benhamou ma.darche emmanuel.pierre frdric.salvy antoine.souliez mo.varroy',FALSE)");

			stmt.executeUpdate("insert into meetings values" + 
			" ('antoine.souliez','runion TPED','dfinition du terme agent','ma.darche'," + 
			"'1/10/1998',1430,4," +
			 "'philippe.benhamou ma.darche emmanuel.pierre frdric.salvy antoine.souliez mo.varroy',FALSE)");

			stmt.executeUpdate("insert into meetings values" + 
			" ('antoine.souliez','installation NIS et CVS','Quelle version de NIS doit-on utiliser ?','ma.darche'," + 
			"'30/01/1999',1030,6," +
			 "'ma.darche antoine.souliez',FALSE)");

			stmt.executeUpdate("insert into meetings values" + 
			" ('ma.darche','runion TPED','dfinition du terme agent','ma.darche'," + 
			"'1/10/1998',1430,4," +
			 "'philippe.benhamou ma.darche emmanuel.pierre frdric.salvy antoine.souliez mo.varroy',FALSE)");

			/* Show what the populating process has done */
			/*
			ResultSet rs1 = stmt.executeQuery("select * from servers");
			while(rs1.next()){
                        System.out.print(rs1.getString("name")+" ");
                        System.out.print(rs1.getString("url")+" ");
                        System.out.print(rs1.getInt("availability")+" ");
                        System.out.print(rs1.getBoolean("master")+" ");
                        System.out.println("\n");
			}

			rs1 = stmt.executeQuery("select * from users");
			while(rs1.next()){
                        System.out.print(rs1.getString("name")+" ");
                        System.out.print(rs1.getString("password")+" ");
                        System.out.print(rs1.getString("info")+" ");
                        System.out.print(rs1.getString("profile")+" ");
                        System.out.print(rs1.getBoolean("online")+" ");
                        System.out.println("\n");
			}

			rs1 = stmt.executeQuery("select * from meetings");
			while(rs1.next()){
                        System.out.print(rs1.getString("user")+" ");
                        System.out.print(rs1.getString("subject")+" ");
                        System.out.print(rs1.getString("note")+" ");
                        System.out.print(rs1.getString("initiator")+" ");
                        System.out.print(rs1.getDate("date")+" ");
                        System.out.print(rs1.getInt("timeofday")+" ");
                        System.out.print(rs1.getInt("duration")+" ");
                        System.out.print(rs1.getString("participants")+" ");
                        System.out.print(rs1.getBoolean("frozen")+" ");
                        System.out.println("\n");
			}

			rs1 = stmt.executeQuery("select * from agents");
			while(rs1.next()){
                        System.out.print(rs1.getString("name")+" ");
                        System.out.print(rs1.getString("owner")+" ");
                        System.out.print(rs1.getString("home")+" ");
                        System.out.print(rs1.getString("currentlocation")+" ");
                        System.out.print(rs1.getBoolean("frozen")+" ");
                        System.out.println("\n");
			}
			*/
			System.out.println("\ntables successfuly created and populated !\n");
			break;
		default:
			System.out.println("\nYou have to chose between 0, 1, 2 and 3 !");
			System.out.println("And remember, use Deletion before any Creation !\n");
			break;
		}//end of switch

	} catch(Exception e){System.out.println(e.toString());}
	}else{
		System.out.println("\nUse Deletion before any Creation !");
		System.out.println("\nYou have to give an argument : 0, 1, 2 or 3 !");
		System.out.println("0 : Deletion");
		System.out.println("1 : Creation");
		System.out.println("2 : Creation and Population\n");
		System.out.println("3 : Creation and Population (Bis)\n");
	     }

}//end of main
}//end of class DBcreation

