
package orchid.server;

import orchid.object.*;
import java.util.Date;
import java.util.Calendar;
import java.util.Vector;
import java.sql.*;

/**
 * A class used by servers to strore and retrieve data to and from the Data Bases of the <b>Orchid</b> system.<P>
 * @author Marc-Aurle DARCHE
 */
public class DBQuerying{
	Statement stmt;

	public DBQuerying(String dbname){
		String url="jdbc:postgresql://localhost:5432/";
		String usr="pr9";
		String pwd="";
		url=url+dbname;
		try{
			Class.forName("postgresql.Driver");
			Connection db = DriverManager.getConnection(url, usr, pwd);
			if(db != null){ System.out.println("Connected to db");
			}else{System.out.println("Can't connect to db");}

			stmt = db.createStatement();
		} catch(Exception e){System.out.println(e.toString());}
	}

	//Part 1.-- SAA Server administration methods

	/**
	 * give all the servers info known by the SAA.
	 */
	public Vector getServers(String filter){
		Vector siList= new Vector();
		try{
			ResultSet rs1 = stmt.executeQuery("select * from servers "+filter);
			while(rs1.next()){
			ServerInfo si=new ServerInfo(rs1.getString("name"),rs1.getString("url"),rs1.getInt("availability"),rs1.getBoolean("master"));
			siList.addElement(si);
			}
		} catch(Exception e){System.out.println(e.toString());}
		return siList;
	}

	/**
	 * add a new server to the servers database of the SAA.
	 */
	public void addServer(ServerInfo si){
		try{
			stmt.executeUpdate("insert into servers values ('"+
				si.getName()+"','"+si.getUrl()+"',"+si.getAvailability()+","+si.getMaster()+")");
		} catch(Exception e){System.out.println(e.toString());}
	}

	/**
	 * remove a server from the servers database of the SAA.
	 */
	public void removeServer(ServerInfo si){
		try{
			stmt.executeUpdate("delete from servers where url='"+si.getUrl()+"'");
		} catch(Exception e){System.out.println(e.toString());}
	}

	/**
	 * update info about a server in the servers database of the SAA.
	 */
	public void updateServer(ServerInfo si){
		try{
			stmt.executeUpdate("update servers set url='"+si.getUrl()+"', availability='"+si.getAvailability()+"', master='"+si.getMaster()+
						"' where url='"+si.getUrl()+"'");
		} catch(Exception e){System.out.println(e.toString());}
		System.out.println("Information about Server si updated !");
	}

	/**
	 * Tells wether a user authentified by a login and a password is authorized to log in.
	 */
	public boolean getAuthentication(String login, String password){
		boolean found= false;
		try{
			ResultSet rs1 = stmt.executeQuery("select name, password from users");
			while(rs1.next()){
				if ((login.equals(rs1.getString("name"))) && 
				    (password.equals(rs1.getString("password")))) found= true;
			}
		} catch(Exception e){System.out.println(e.toString());}
		return found;
	}


	//Part 2.-- SAA Agent Administration methods

	/**
	 * give all the agents info living on the local server.
	 */
	public Vector getAgents(String filter){
		Vector aiList= new Vector();
		try{
			ResultSet rs1 = stmt.executeQuery("select * from agents "+filter);
			while(rs1.next()){
			AgentInfo ai=new AgentInfo(rs1.getString("name"),rs1.getString("owner"),rs1.getString("home"),rs1.getString("currentlocation"),rs1.getBoolean("frozen"));
			aiList.addElement(ai);
			}
		} catch(Exception e){System.out.println(e.toString());}
		return aiList;
	}

	/**
	 * add an agent to the agents database of the SAA.
	 */
	public void addAgent(AgentInfo ai){
		try{
			stmt.executeUpdate("insert into agents values ('"+
				ai.getName()+"','"+ai.getOwner()+"','"+ai.getHome()+"','"+ai.getCurrentlocation()+"',"+ai.getFrozen()+")");
		} catch(Exception e){System.out.println(e.toString());}
	}

	/**
	 */
	public void removeAgent(AgentInfo ai){
		try{
			stmt.executeUpdate("delete from agents where name='"+ai.getName()+"'");
		} catch(Exception e){System.out.println(e.toString());}
	}

	/**
	 */
	public void updateAgent(AgentInfo ai){
		try{
			stmt.executeUpdate("update agents set currentlocation='"+ai.getCurrentlocation()+"', frozen='"+ai.getFrozen()+"' where name='"+ai.getName()+"'");
		} catch(Exception e){System.out.println(e.toString());}
	}


	//Part 3.-- Meeting querying methods

	/**
	 * This method return a Vector, <B>sorted by date</B>, containing the Meetings scheduled <B>between the beginning date and the end date</B>.
	 */
	public Vector getMeetings(String user, Date beginning, Date end){
		Vector mList= new Vector();
		try{
			java.sql.Date sqlbeginning= new java.sql.Date(beginning.getTime());
			java.sql.Date sqlend= new java.sql.Date(end.getTime());

			ResultSet rs1 = stmt.executeQuery("select * from meetings where user='"+user+"' and date >= '"+sqlbeginning+"' and date <= '"+sqlend+"' order by date");
			while(rs1.next()){
				Calendar cal= Calendar.getInstance();
				cal.setTime(rs1.getDate("date"));
				int year=cal.get(Calendar.YEAR);
				int month=cal.get(Calendar.MONTH);
				int day=cal.get(Calendar.DAY_OF_MONTH);
				Vector participants= Util.stringToVector(rs1.getString("participants"));
				Meeting m=new Meeting(rs1.getString("subject"),rs1.getString("note"),rs1.getString("initiator"),year,month,day,rs1.getInt("timeofday"),rs1.getInt("duration"),participants,rs1.getBoolean("frozen"));
				mList.addElement(m);
			}
		} catch(Exception e){System.out.println(e.toString());}
		return mList;
	}

	public void addMeeting(String user, Meeting m){
		try{
			stmt.executeUpdate("insert into meetings values ('"+
				user+"','"+m.getSubject()+"','"+m.getNote()+"','"+m.getInitiator()+"','"+
				m.getDate()+"',"+m.getTimeOfDay()+",'"+m.getDuration()+"','"+m.getParticipants()+"','"+m.getFrozen()+"')");
		} catch(Exception e){System.out.println(e.toString());}
	}

	public void removeMeeting(String user, Meeting m){
		try{
			stmt.executeUpdate("delete from meetings where name='"+user+"' and subject='"+m.getSubject()+"' and initiator='"+m.getInitiator()+"'");
		} catch(Exception e){System.out.println(e.toString());}
	}

	public void updateMeeting(Meeting m){
		try{
			stmt.executeUpdate("update meetings set initiator='"+m.getInitiator()+"' where subject='"+m.getSubject()+"'");
		} catch(Exception e){System.out.println(e.toString());}
	}



	/**
	 * The main method is only here to test the other DBQuerying methods.
	 */
	public static void main(String[] args){
		//instanciation of DBQuerying
		DBQuerying dbq=new DBQuerying("orchid_db");

		//examples with ServerInfo related methods
		ServerInfo si=new ServerInfo("i19.esiea.fr","192.168.1.290",2,false);
		ServerInfo si2=new ServerInfo("nef.esiea.fr","192.168.1.103",4,false);
		dbq.addServer(si);
		dbq.removeServer(si2);
		si.setAvailability(30);
		dbq.updateServer(si);

		Vector siList;
		siList=dbq.getServers("");
		System.out.println(siList);
		System.out.println(siList.size());
		ServerInfo si4=(ServerInfo)siList.elementAt(0);
		System.out.println(si4.getName());


		//examples with Authentication 
		AgentInfo ai=new AgentInfo("ma__emmanuel.pierre","emmanuel.pierre","192.168.1.290","192.168.1.103",false);
		System.out.println("antoine.souliez authentified ? "+dbq.getAuthentication("antoine.souliez","q$;s"));
		System.out.println("ma.darche authentified ? "+dbq.getAuthentication("ma.darche","az"));
		System.out.println("bal.tazar authentified ? "+dbq.getAuthentication("bal.tazar","99az"));

		
		//examples with AgentInfo related methods
		AgentInfo ai2=new AgentInfo("ma__madarche","ma.darche","192.168.1.105","192.168.1.105",false);
		dbq.addAgent(ai);
		dbq.removeAgent(ai2);
		ai.setCurrentlocation("192.168.1.105"); ai.setFrozen(true);
		dbq.updateAgent(ai);

		Vector aiList;
		aiList=dbq.getAgents("");
		System.out.println(aiList);
		System.out.println(aiList.size());
		AgentInfo ai4=(AgentInfo)aiList.elementAt(0);
		System.out.println(ai4.getName());


		//examples with Meeting related methods
		Vector mList;
		Calendar cal= Calendar.getInstance();
		cal.add(Calendar.DATE,-300);
		System.out.println("date dbut: "+cal.getTime());
		Date beginningDate= cal.getTime();
		//cal.add(Calendar.DATE,10);
		cal.add(Calendar.DATE,600);
		System.out.println("date fin: "+cal.getTime());
		Date endDate= cal.getTime();
		mList=dbq.getMeetings("philippe.benhamou",beginningDate,endDate);
		System.out.println("Rcupration des meetings entre dbut et fin: "+mList);
		System.out.println("Nombre de meetings: "+mList.size());
		if (mList.size()>0){
			Meeting m4=(Meeting)mList.elementAt(0);
			System.out.println(m4.getSubject());
		}

		Vector v=Util.stringToVector("antoine.souliez aergralserzr e33333zrzer ezrr  32R453.erzze");
		System.out.println("Voici le vecteur v: "+v);
		System.out.println("Voici la taille de v: "+v.size());
		String mot=(String)v.elementAt(2);
		System.out.println("Voici le mot: "+mot);

		System.out.println("Voici la liste de type sql: "+Util.vectorToSQLList(v));
		
	}


}//end of public class DBQuerying

